/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is
 * Henrik Gemal http://gemal.dk
 *
 * Portions created by the Initial Developer are Copyright (C) 2001-2004
 * the Initial Developer. All Rights Reserved.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

// This is a list of ids to elements of the following type: radiogroup, textbox,
// checkbox, menulist that can be prefilled automatically from preferences (default
// or user) by the Options dialog's framework. To benefit from this prefilling each
// checkbox etc that you use should be annotated with the pref identifier that
// it is tied to, the type of pref, and a unique id, which is added to this array.
//
// e.g for this XUL element:
//
// <checkbox id="showSampleWindow" label="Show Sample Window"
//					preftype="bool" prefstring="sample.options.showSampleWindow"/>
//
// _elementIDs would look like this:
//
// var _elementIDs = ["showSampleWindow"];
//
var gLaunchyApps = null;
var gLaunchyAppsList = null;

// This function is called before the dialog is shown, and before the preferences
// auto-filling code has initialized the state of any of the UI elements in this
// dialog. Thus it is not possible to do enabling or disabling at this point since
// you won't correctly know the state of your UI.
function onLoad() {
	// We ask the parent dialog (which is the Firebird Options dialog) to initialize
	// this by using the preferences auto-prefill code.
	if (typeof(window.opener.top.initPanel) != "undefined") {
		window.opener.top.initPanel(window.location.href, window);
	} else {
		Startup();
	}
}

// This is a special function that is called by the preferences auto-prefilling code
// AFTER all of the UI elements defined in _elementIDs above have been prefilled from
// the user or default preferences. You can execute code in this method that enables
// or disables elements based on the state of various UI elements, since their state
// has already been established.
function Startup() {
	if (LaunchyContext.jslib && LaunchyContext.jslibversion) {
		var label = document.getElementById("bundle_launchy").getString("launchy-version-jslib-info");
		document.getElementById("launchy-version-jslib").setAttribute("value", launchyShared.replaceInsert(label, 1, LaunchyContext.jslibversion));
		var ld = LaunchyContext.fu.append(LaunchyContext.du.getChromeDir(), "launchy.jar");
		if (LaunchyContext.fu.exists(ld)) {
			document.getElementById("launchy-location").setAttribute("hidden", false);
			document.getElementById("launchy-location-path").setAttribute("value", ld);
		} else {
			ld = LaunchyContext.fu.append(LaunchyContext.du.getUserChromeDir(), "launchy.jar");
			if (LaunchyContext.fu.exists(ld)) {
				document.getElementById("launchy-location").setAttribute("hidden", false);
				document.getElementById("launchy-location-path").setAttribute("value", ld);
			}
		}
	} else {
		document.getElementById("launchy-version-jslib").setAttribute("value", document.getElementById("bundle_launchy").getString("launchy-version-jslib-info-no"));
	}
	gLaunchyApps = new LaunchyApps();
	gLaunchyAppsList = document.getElementById("launchyAppsList");
	gLaunchyAppsList.treeBoxObject.view = gLaunchyApps;
}

// The user pressed the OK button on the dialog.
function onOK() {
	// Tell the preferences framework to save the user's modifications for this
	// panel, but don't actually save them to disk until the user presses "OK" in
	// the master Options dialog.
	window.opener.top.hPrefWindow.wsm.savePageData(window.location.href, window);

	// Dialog OK handlers must return true.
	return true;
}

// Any specialized enabling code and code for other UI controls in the options dialog
// goes here.

function GetTreeSelections(tree) {
	var selections = [];
	var select = tree.view.selection;
	if (select) {
		var count = select.getRangeCount();
		var min = new Object();
		var max = new Object();
		for (var i=0; i<count; i++) {
			select.getRangeAt(i, min, max);
			for (var k=min.value; k<=max.value; k++) {
				if (k != -1) {
					selections[selections.length] = k;
				}
			}
		}
	}
	return selections;
}

function LaunchyAppSelected() {
	var selections = GetTreeSelections(gLaunchyAppsList);

	var idx = selections[0];
	var props = [
		{id: "launchy-apps-info-name", value: gLaunchyApps._LaunchyApps[idx].appName},
		{id: "launchy-apps-info-type", value: LaunchyContext.appstype[gLaunchyApps._LaunchyApps[idx].appType]},
		{id: "launchy-apps-info-exe", value: gLaunchyApps._LaunchyApps[idx].appExe},
		{id: "launchy-apps-info-args", value: gLaunchyApps._LaunchyApps[idx].appArgs},
	];

	if (idx >= gLaunchyApps._LaunchyApps.length) {
		launchyShared.trace("Tree and viewer state are out of sync! Help us figure out the problem in bug 119812\nidx: " + idx + " - gLaunchyApps._LaunchyApps.length: " + LaunchyContext.appstype.length, 1);
		return true;
	}

	var field;
	for (var i = 0; i < props.length; i++) {
		field = document.getElementById(props[i].id);
		field.value = props[i].value;
	}
	return true;
}

function LaunchyApp(appName, appType, appExe, appArgs) {
	this.appName = appName;
	this.appType = appType;
	this.appExe = appExe;
	this.appArgs = appArgs;
}

function LaunchyApps() {
	this._LaunchyApps = [];
	if (typeof(LaunchyContext.apps) == "undefined") {
		return;
	} else {
		var atomSvc = Components.classes["@mozilla.org/atom-service;1"].getService(Components.interfaces.nsIAtomService);
		this._enabledAtom = atomSvc.getAtom("enabled");
		for (var j = 0; j < LaunchyContext.apps.length; j++) {
			this.addType(LaunchyContext.apps[j].name, LaunchyContext.apps[j].type, LaunchyContext.apps[j].exe, LaunchyContext.apps[j].args);
		}
		for (j = 0; j < LaunchyContext.appsnot.length; j++) {
			this.addType(LaunchyContext.appsnot[j].name, LaunchyContext.appsnot[j].type, LaunchyContext.appsnot[j].exe, LaunchyContext.appsnot[j].args);
		}
	}
}

LaunchyApps.prototype = {
	addType: function (appName, appType, appExe, appArgs) {
		var pluginType = new LaunchyApp(appName, appType, appExe, appArgs);
		this._LaunchyApps.push(pluginType);
	},

	get rowCount() {
		return this._LaunchyApps.length;
	},

	getCellProperties: function (row, column, aProperties) {
		switch(column.id || column) {
			case "appEnabled":
			case "appName":
				if (this._LaunchyApps[row].appExe)
					aProperties.AppendElement(this._enabledAtom);
		}
	},

	getCellText: function (row, column) {
		switch(column.id || column) {
			case "appType":
				return LaunchyContext.appstype[this._LaunchyApps[row].appType];

			case "appName":
				return this._LaunchyApps[row].appName;

			default:
				return "";
		}
	},

	getImageSrc: function (row, column) {
		switch(column.id || column) {
			case "appName":
				if (this._LaunchyApps[row].appExe)
					return "moz-icon:file://" + this._LaunchyApps[row].appExe + "?size=16";
				else
					return null;

			default:
				return null;
		}
	},

	selection: null,

	// Stuff we don't need...
	isContainer: function (aRow) { return false; },
	isContainerOpen: function (aRow) { return false; },
	isContainerEmpty: function (aRow) { return true; },
	isSeparator: function (aRow) { return false; },
	isSorted: function () { return false; },
	canDrop: function (aRow, aOrientation) { return false; },
	drop: function (aRow, aOrientation) { },
	getParentIndex: function (aRow) { },
	hasNextSibling: function (aRow, aNextIndex) { },
	getLevel: function (aRow) { },
	getProgressMode: function (aRow, aCol) { },
	getCellValue: function (aRow, aCol) { },
	cycleCell: function (aRow, aCol) { },
	setTree: function (aTree) { },
	toggleOpenState: function (aRow) { },
	cycleHeader: function (aCol) { },
	selectionChanged: function () { },
	isEditable: function (aRow, aCol) { },
	setCellValue: function (aRow, aCol, aValue) { },
	setCellText: function (aRow, aCol, aValue) { },
	performAction: function (aAction) { },
	performActionOnRow: function (aAction, aRow) { },
	performActionOnCell: function (aAction, aRow, aCol) { },
	getRowProperties: function (aRow, aProperties) { },
	getColumnProperties: function (aCol, aProperties) { }
};
